#!/bin/bash
#
# Copyright(c) 2016 System Fabric Works, Inc.
#
# Set cpu governors to performance.
#

set -e
GOV=performance
if [ "$1" == "-n" ] ; then
    GOV=ondemand
fi

if which cpupower; then
    cpupower frequency-set -g $GOV
elif which cpufreq-set; then
    for cpu in $(seq 1 `grep MHz /proc/cpuinfo | wc -l`); do 
        cpufreq-set -c `expr $cpu - 1` -g $GOV
    done
else
    echo "Could not find cpupower or cpufreq-set in path"
    exit 1
fi

if [ $GOV == "ondemand" ] ; then
	exit
fi

mhz=
rc=0
for val in `grep MHz /proc/cpuinfo | cut -f2 -d:`; do
    if [ -z "$mhz" ] ; then
	mhz=$val
    elif [ $val != $mhz ] ; then
	echo "WARNING: Inconsistent MHz: $val"
	rc=1
    fi
done

if [ $rc -ne 0 ] ; then
    echo "You may need to add intel_pstate=disable to the kernel command line."
fi

exit $rc
	
